/**
 *  Copyright (C) 2009  Thomas Volk
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package net.sf.gtools.jms

import javax.jms.ConnectionFactory
import javax.jms.Connection
import javax.jms.Destination
import javax.jms.Topic
import javax.jms.Queue
import javax.jms.Session
import javax.jms.MessageConsumer
import javax.jms.MessageProducer
import javax.jms.JMSException
import javax.jms.MapMessage
import javax.jms.Message

class JmsCategory {

    static String getName(Queue destination) {
    	return destination.getQueueName()
    }

    static String getName(Topic destination) {
    	return destination.getTopicName()
    }
    
    static Map toMap(MapMessage message) {
    	Map result = [:] as LinkedHashMap
    	message.getMapNames().each{
    		key -> result[key] = message.getObject(key) }
		return result
    }
    
    static MapMessage createMapMessage(Session session, Map map) {
    	MapMessage message = session.createMapMessage()
		map.each { key, val -> message.setObject(key, val)}
		return message
    }
    
    static Session createSession(Connection connection, Map param = [:]) {
		boolean isTransacted = param.get("transacted", false) as Boolean
		int mode = param.get("mode", Session.AUTO_ACKNOWLEDGE) as Integer
		return connection.createSession(isTransacted, mode)
    }
    
    static void connect(ConnectionFactory factory, Closure closure) {
        def connection = factory.createConnection()
        connection.start()
        try {
            closure(connection)
        }
        finally {
            connection.close()
        }
    }

	static void session(Connection connection, Map param=[:], Closure closure) {
		def session = connection.createSession(param)
		try {
			try {
                closure(session)
                if (session.transacted) {
                        session.commit()
                }
			} catch (JMSException e) {
				rollback(session)
				throw e
			} catch (RuntimeException e) {
				rollback(session)
				throw e
			} catch (Exception e) {
				rollback(session)
				throw e
			}
		} finally {
			session.close();
		}
    }
	
	static private void rollback(Session session) {
		if (session.transacted) {
			session.rollback()
		}
	}
	   
	static void topic(Session session, String name, Map param=[:],
			Closure inClosure) {
		param.put("type", JmsDestinationType.TOPIC)
		destination(session, name, param, inClosure)
	}
	   
	static void queue(Session session, String name, Map param=[:],
			Closure inClosure) {
		param.put("type", JmsDestinationType.QUEUE)
		destination(session, name, param, inClosure)
	}
	
	static void destination(Session session, String name, Map param=[:],
			Closure closure) {
		def jmsDestination = new JmsDestination(session,
				session.createDestination(name, param))
		closure.delegate = session
		closure(jmsDestination)
	}
    
    static Destination createDestination(Session session, String name,
    		Map param) {
    	def type = param.get("type", JmsDestinationType.QUEUE) as JmsDestinationType
    	return type.create(session, name)
	}
	
    static void producer(JmsDestination jmsDestination,
            Closure inClosure) {
	def producer = jmsDestination.createProducer()
	    try {
            inClosure(producer)
        }
    	finally {
            producer.close()
    	}
    }
	
    static void consumer(JmsDestination jmsDestination, Map param=[:],
    		Closure inClosure) { 	
    	def consumer = jmsDestination.createConsumer(param)
    	try {
            inClosure(consumer)
        }
    	finally {
            consumer.close()
    	}
    }
 
    static MessageConsumer createConsumer(Session session,
    		Destination inDestination, Map inParam) {
    	def selector = inParam.get("selector")
		boolean noLocal = inParam.get("noLocal", false) as Boolean
    	return session.createConsumer(inDestination, selector, noLocal)
    }
    
    
    static void queue(Connection connection, String name, Map param=[:],
    		Closure closure) {
    	connection.session(param) { session ->
            session.queue(name, param) { jmsDestination ->
                def jmsClient = new JmsClient(jmsDestination, param)
                closure.delegate = session
                closure(jmsClient)
            }
    	}
    }

    static void topic(Connection connection, String name, Map param=[:],
    		Closure closure) {
    	connection.session(param) { session ->
    	    session.topic(name, param) { jmsDestination ->
                def jmsClient = new JmsClient(jmsDestination, param)
                closure.delegate = session
                closure(jmsClient)
            }
    	}
    }
    
    static void send(Session session, Destination dest, Message msg) {
        def jmsDest = new JmsDestination(session, dest)
        jmsDest.producer { producer ->
            producer.send(msg)
        }
    }
    
    static void replyTo(Session session, Message msg, Message response) {
        response.setJMSCorrelationID(msg.getJMSMessageID())
        session.send(msg.getJMSReplyTo(), response)
    }
    
    static void setJMSReplyTo(Message msg, JmsDestination dest) {
        msg.setJMSReplyTo(dest.destination)
    }
    
    static void setJMSReplyTo(Message msg, JmsClient client) {
        msg.setJMSReplyTo(client.destination.destination)
    }
}
